/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelconfigwinclass.c,v 1.8 2009/09/27 19:18:06 kiero Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "ambient_cat.h"
#include "methodstack.h"
#include "mui_func.h"
#include "screen.h"
#include "legacy.h"


struct Data {
	APTR parentobj;
	APTR bt_zip;
	APTR txt_zip;
	APTR bt_autozip;
	APTR txt_autozip;
	APTR cyc_placement;
};


ULONG sizeslidera[] = {
	MV_Panelgroup_Size_Micro,
	MV_Panelgroup_Size_Small,
	MV_Panelgroup_Size_Medium,
	MV_Panelgroup_Size_Large,
	MV_Panelgroup_Size_Huge,
	NULL
};


static ULONG panel_size_to_slider(ULONG size)
{
	ULONG *p = sizeslidera;
	ULONG i = 0;

	while (*p)
	{
		if (*p == size)
		{
			break;
		}
		i++;
		p++;
	}
	return (i);
}


static ULONG panel_slider_to_size(ULONG slider)
{
	return (sizeslidera[slider]);
}


static ULONG panel_depth_to_cycle(APTR parentobj)
{
	return getv(parentobj, MA_Panelgroup_Depth);
}


static void panel_cycle_to_depth(APTR parentobj, ULONG cycleval)
{
	APTR winobj = _win(parentobj);

	if (winobj) set(winobj, MUIA_Window_Open, FALSE);

	set(parentobj, MA_Panelgroup_Depth, cycleval);

	if (winobj) set(winobj, MUIA_Window_Open, TRUE); /* XXX: what happens if the window doesn't open again ? */
}


static ULONG panel_dragmode_to_cycle(ULONG dragmode)
{
	switch (dragmode)
	{
		case MV_Panelgroup_DragMode_None:
			return (0);

		case MV_Panelgroup_DragMode_LeftUp:
			return (1);

		case MV_Panelgroup_DragMode_RightBottom:
			return (2);

		case MV_Panelgroup_DragMode_Both:
			return (3);
	}
	PDB(("huho.. nodragmode %ld\n", dragmode));

	return (0);
}

static ULONG panel_cycle_to_dragmode(ULONG cycle)
{
	switch (cycle)
	{
		case 0:
			return (MV_Panelgroup_DragMode_None);

		case 1:
			return (MV_Panelgroup_DragMode_LeftUp);

		case 2:
			return (MV_Panelgroup_DragMode_RightBottom);

		case 3:
			return (MV_Panelgroup_DragMode_Both);
	}
	PDB(("huho.. no cyclemode %ld\n", cycle));

	return (0);
}

/*  NOTE: can be merged to OM_NEW as soon we deprecate MUI3.9 support completly
 */
static APTR panel_create_frimageselector(APTR parentobj)
{
	APTR pop_frimage = pop_frimage = MUI_NewObject("Popfrimage.mui", // MUIC_Popfrimage,
		MUIA_Window_Title, GSI(MSG_PANELCFG_ADJUST_BACKGROUND_FRAMES),
		MUIA_Framedisplay_Spec,	getv(parentobj, MA_Panelgroup_FrameSpec),
		MUIA_Imagedisplay_Spec, getv(parentobj, MA_Panelgroup_ImageSpec),
		MUIA_CycleChain, 1,
	End;

	if (pop_frimage)
	{
		set(pop_frimage, MUIA_ShortHelp, GSI(MSG_PANELCFG_ADJUST_BACKGROUND_FRAMES_HELP));
			DoMethod(pop_frimage, MUIM_Notify, MUIA_Framedisplay_Spec, MUIV_EveryTime,
			MUIV_Notify_Window, 2, MM_Panelconfigwin_FrameSpec, MUIV_TriggerValue
		);
	
		DoMethod(pop_frimage, MUIM_Notify, MUIA_Imagedisplay_Spec, MUIV_EveryTime,
			MUIV_Notify_Window, 2, MM_Panelconfigwin_ImageSpec, MUIV_TriggerValue
		);

		DoMethod(pop_frimage, MUIM_Notify, MUIA_PopClosing, TRUE,
			MUIV_Notify_Window, 1, MM_Panelconfigwin_SaveSpec
		);

		return pop_frimage;
	}

	return NULL;
}


DEFNEW
{
	struct Data *data;
	struct TagItem *ti;
	APTR cyc_ori, cyc_pos, cyc_placement, cyc_depth;
	APTR bt_zip, bt_autozip;
	APTR txt_zip, txt_autozip;
	APTR sl_size;
	APTR parentobj;
	ULONG dragmode;

	static STRPTR cyc_modes[3];
	static STRPTR cyc_titles[3];
	static STRPTR cyc_depths[4];

	static STRPTR cyc_placementh[5];

//	  cyc_titles[0] = GSI(MSG_PANELCFG_TITLES_VERTICAL);
//	  cyc_titles[1] = GSI(MSG_PANELCFG_TITLES_HORIZONTAL);

//	  cyc_modes[0] = GSI(MSG_PANELCFG_MODES_FLOATING);
//	  cyc_modes[1] = GSI(MSG_PANELCFG_MODES_ATTACHED);

	cyc_depths[0] = GSI(MSG_PANELCFG_DEPTHS_NORMAL);
	cyc_depths[1] = GSI(MSG_PANELCFG_DEPTHS_BACK);
	cyc_depths[2] = GSI(MSG_PANELCFG_DEPTHS_FRONT);
	
//	  cyc_placementh[0] = GSI(MSG_PANELCFG_PLACEMENTH_NONE);
//	  cyc_placementh[1] = GSI(MSG_PANELCFG_PLACEMENTH_BEGINNING);
//	  cyc_placementh[2] = GSI(MSG_PANELCFG_PLACEMENTH_END);
//	  cyc_placementh[3] = GSI(MSG_PANELCFG_PLACEMENTH_BOTH);
	
	if ((ti = FindTagItem(MA_Panelconfigwin_Parent, INITTAGS)))
	{
		parentobj = (APTR)ti->ti_Data;
	}
	else
	{
		return (NULL);
	}

	obj = DoSuperNew(cl, obj,
		MUIA_Window_Screen,      get_screen(),
		MUIA_Window_ScreenTitle, screentitle,
		MUIA_Window_Title,       GSI(MSG_PANELCFG_WINDOW_TITLE),
		MUIA_Window_LeftEdge,    getv(_win(parentobj), MUIA_Window_LeftEdge) + getv(_win(parentobj), MUIA_Window_Width),//MUIV_Window_LeftEdge_Moused,
		MUIA_Window_TopEdge,     getv(_win(parentobj), MUIA_Window_TopEdge),//MUIV_Window_TopEdge_Moused,
		MUIA_Window_Width,       MUIV_Window_Width_MinMax(10),
		MUIA_Window_Height,      MUIV_Window_Height_MinMax(10),
		MUIA_Window_ShowIconify, FALSE,
		MUIA_Window_ShowPrefs,   FALSE,
		MUIA_Window_ShowJump,    FALSE,
		MUIA_Window_ShowAbout,   FALSE,
		MUIA_Window_NoMenus,     TRUE,

		WindowContents, VGroup,
		
			Child, VSpace(-1),

			Child, HGroup,

				Child, HSpace(-1),

				Child, ColGroup(2),
							
					Child, NSLabel(MSG_PANELCFG_ORIENTATION),
					Child, cyc_ori = MUICreateCycle( MSG_PANELCFG_ORIENTATION, cyc_titles, MSG_PANELCFG_TITLES_VERTICAL, MSG_PANELCFG_TITLES_HORIZONTAL, "PANEORIE"),

					Child, NSLabel(MSG_PANELCFG_POSITIONING),
					Child, cyc_pos = MUICreateCycle( MSG_PANELCFG_POSITIONING, cyc_modes, MSG_PANELCFG_MODES_FLOATING, MSG_PANELCFG_MODES_ATTACHED, "PANEPOSI"),

					Child, NSLabel(MSG_PANELCFG_SIZE),
					Child, sl_size = NewObject(getpanelslidersizeclass(), NULL, MUIA_ControlChar, MUIGetUnderScore( MSG_PANELCFG_SIZE ), MUIA_ShortHelp, GSI(MSG_PANELCFG_SIZE_HELP), TAG_DONE),

					Child, NSLabel(MSG_PANELCFG_DEPTH_ARRANGEMENT),
					Child, cyc_depth = MUICreateCycle( MSG_PANELCFG_DEPTH_ARRANGEMENT, cyc_depths, 0L, 0L, "PANEDEPT"),

					Child, NSLabel(MSG_PANELCFG_DRAG_GADGET),
					Child, cyc_placement = MUICreateCycle( MSG_PANELCFG_DRAG_GADGET, cyc_placementh, MSG_PANELCFG_PLACEMENTH_NONE, MSG_PANELCFG_PLACEMENTH_BOTH, "PANEDRAG"),

					Child, txt_zip = NSLabel1(MSG_PANELCFG_ALLOW_ZIPPING),
					Child, HGroup,
						Child, bt_zip = MUICreateCheckbox( MSG_PANELCFG_ALLOW_ZIPPING, FALSE, "PANEZIPP"),
						Child, HSpace(-1),
					End,

					Child, txt_autozip = NSLabel1(MSG_PANELCFG_AUTO_ZIPPING),
					Child, HGroup,
						Child, bt_autozip = MUICreateCheckbox( MSG_PANELCFG_AUTO_ZIPPING, FALSE, "PANEAZIP"),
						Child, HSpace(-1),
					End,

					Child, VGroup,
						Child, VSpace(-1),
						Child, HGroup,
							Child, HSpace(-1),
							Child, NSLabel(MSG_PANELCFG_LOOK),
						End,
						Child, VSpace(-1),
					End,
					
					Child, panel_create_frimageselector(parentobj),
		
				End,

				Child, HSpace(-1),
			
			End,

			Child, VSpace(-1),
		End,
	
	End;

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);
	data->parentobj = parentobj;
	data->cyc_placement = cyc_placement;
	data->bt_zip = bt_zip;
	data->txt_zip = txt_zip;
	data->bt_autozip = bt_autozip;
	data->txt_autozip = txt_autozip;

	DoMethod(obj, MUIM_MultiSet, MUIA_CycleChain, 1,
		cyc_ori, cyc_pos, sl_size, cyc_depth, cyc_placement, NULL
	);

	set(cyc_pos,   MUIA_Cycle_Active, getv(_win(parentobj), MA_Panelwin_Attached));
	set(sl_size,   MUIA_Slider_Level, panel_size_to_slider(getv(parentobj, MA_Panelgroup_Size)));
	set(cyc_depth, MUIA_Cycle_Active, panel_depth_to_cycle(parentobj));
	set(cyc_ori,   MUIA_Cycle_Active, getv(parentobj, MA_Panelgroup_Horiz));

	if ((dragmode = getv(parentobj, MA_Panelgroup_DragMode)))
	{
		set(cyc_placement, MUIA_Cycle_Active, panel_dragmode_to_cycle(dragmode));
	}
	DoMethod(obj, MM_Panelconfigwin_DragMode);
	
	DoMethod(bt_zip, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_Zipping, MUIV_TriggerValue
	);

	DoMethod(bt_autozip, MUIM_Notify, MUIA_Selected, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_AutoZip, MUIV_TriggerValue
	);

	set(bt_zip, MUIA_Selected, getv(data->parentobj, MA_Panelgroup_Zipping));
	set(bt_autozip, MUIA_Selected, getv(data->parentobj, MA_Panelgroup_AutoZip));

	DoMethod(obj, MUIM_Notify, MUIA_Window_CloseRequest, TRUE,
		obj, 1, MM_Panelconfigwin_Close
	);

	DoMethod(cyc_pos, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_PosMode, MUIV_TriggerValue
	);

	DoMethod(cyc_placement, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
		obj, 1, MM_Panelconfigwin_DragMode
	);

	DoMethod(sl_size, MUIM_Notify, MUIA_Slider_Level, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_Size, MUIV_TriggerValue
	);

	DoMethod(cyc_depth, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_Depth, MUIV_TriggerValue
	);

	DoMethod(cyc_ori, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime,
		obj, 2, MM_Panelconfigwin_Horiz, MUIV_TriggerValue
	);

	return ((ULONG)obj);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Window_ID:
			*msg->opg_Storage = 0;
			return (TRUE);

		case MA_Window_Path:
			*msg->opg_Storage = MV_Window_Path_None;
			return (TRUE);

		case MA_Window_Type:
			*msg->opg_Storage = MV_Window_Type_Panelconfig;
			return (TRUE);

		case MA_Window_UserData:
			{
				GETDATA;
				*msg->opg_Storage = (ULONG)data->parentobj;
			}
			return (TRUE);
	}
	return (DOSUPER);
}


DEFTMETHOD(Panelconfigwin_Close)
{
	GETDATA;

	if (data->parentobj)
	{
		set(data->parentobj, MA_Panelgroup_Configwin, NULL);
	}

	methodstack_push(_app(obj), 2, MM_Application_DisposeWindow, obj);

	return (0);
}


DEFTMETHOD(Panelconfigwin_DragMode)
{
	ULONG disable;
	GETDATA;

	disable = getv(data->cyc_placement, MUIA_Cycle_Active) ? FALSE : TRUE;

	DoMethod(obj, MUIM_MultiSet, MUIA_Disabled, disable,
		data->txt_zip, data->bt_zip,
		NULL
	);

	if (!disable)
	{
		/* Disable/enable autozipping button */

		disable = !getv(data->bt_zip, MUIA_Selected);

		DoMethod(obj, MUIM_MultiSet, MUIA_Disabled, disable,
			data->txt_autozip, data->bt_autozip,
			NULL
		);

		set(data->parentobj, MA_Panelgroup_DragMode, panel_cycle_to_dragmode(getv(data->cyc_placement, MUIA_Cycle_Active)));
	}
	else
	{
		DoMethod(obj, MUIM_MultiSet, MUIA_Disabled, disable,
			data->txt_autozip, data->bt_autozip,
			NULL
		);

		set(data->parentobj, MA_Panelgroup_DragMode, MV_Panelgroup_DragMode_None);
	}

	return (0);
}


DEFSMETHOD(Panelconfigwin_Zipping)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_Zipping, msg->selected);
	set(data->bt_autozip, MUIA_Disabled, !msg->selected);
	set(data->txt_autozip, MUIA_Disabled, !msg->selected);

	return (0);
}


DEFSMETHOD(Panelconfigwin_AutoZip)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_AutoZip, msg->selected);

	return (0);
}


DEFSMETHOD(Panelconfigwin_Size)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_Size, panel_slider_to_size(msg->size));

	return (0);
}


DEFSMETHOD(Panelconfigwin_Depth)
{
	GETDATA;

	panel_cycle_to_depth(data->parentobj, msg->depth);

	return (0);
}


DEFSMETHOD(Panelconfigwin_Horiz)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_Horiz, msg->horiz);

	return (0);
}


DEFSMETHOD(Panelconfigwin_FrameSpec)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_FrameSpec, msg->spec);

	return (0);
}


DEFSMETHOD(Panelconfigwin_ImageSpec)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_ImageSpec, msg->spec);

	return (0);
}


DEFTMETHOD(Panelconfigwin_SaveSpec)
{
	GETDATA;

	set(data->parentobj, MA_Panelgroup_HasChanged, TRUE);

	return (0);
}


DEFSMETHOD(Panelconfigwin_PosMode)
{
	GETDATA;

	switch (msg->mode)
	{
		case MV_Panelconfig_PosMode_Floating:
			if (muiRenderInfo(obj) && _win(data->parentobj))
			{
				set(_win(data->parentobj), MA_Panelwin_Attached, FALSE);
			}
			break;

		case MV_Panelconfig_PosMode_Attached:
			if (muiRenderInfo(obj) && _win(data->parentobj))
			{
				set(_win(data->parentobj), MA_Panelwin_Attached, TRUE);
			}
			break;

		#ifdef DEBUG
		default:
			PDB(("wrong posmode\n"));
			break;
		#endif
	}
	return (0);
}


BEGINMTABLE
DECNEW
DECGET
DECTMETHOD(Panelconfigwin_Close)
DECTMETHOD(Panelconfigwin_DragMode)
DECSMETHOD(Panelconfigwin_Zipping)
DECSMETHOD(Panelconfigwin_AutoZip)
DECSMETHOD(Panelconfigwin_Size)
DECSMETHOD(Panelconfigwin_Depth)
DECSMETHOD(Panelconfigwin_Horiz)
DECSMETHOD(Panelconfigwin_FrameSpec)
DECSMETHOD(Panelconfigwin_ImageSpec)
DECTMETHOD(Panelconfigwin_SaveSpec)
DECSMETHOD(Panelconfigwin_PosMode)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Window, panelconfigwinclass)
