/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: paneldragclass.c,v 1.8 2006/12/20 13:34:15 fab Exp $
 */

#include "ambient.h"

/* public */
#include <intuition/gadgetclass.h>
#include <proto/graphics.h>
#include <cybergraphx/cybergraphics.h>
#include <proto/cybergraphics.h>


/* private */
#include "mui_func.h"
#include "legacy.h"
#include "gfx_analyze.h"



struct Data {
	APTR draggad;
	struct MUI_EventHandlerNode ehnode;
	ULONG type;
	ULONG selected;
	ULONG locked;
	UBYTE *tbuf;
	ULONG oldbufsize;
	ULONG oldx;
	ULONG oldy;
	APTR objdown;
	ULONG zipping;
};

#define MINEMPTY 12


DEFNEW
{
	struct Data *data;
	struct TagItem *ti;

	obj = DoSuperNew(cl, obj,
		MUIA_DoubleBuffer, TRUE,
	End;

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);
	
	if ( (ti = FindTagItem(MA_Paneldrag_Type, INITTAGS)) )
	{
		data->type = ti->ti_Data;
	}
	#ifdef DEBUG
	else
	{
		PDB(("argh! no MA_Paneldrag_Type given\n"));
	}
	#endif

	return ((ULONG)obj);
}


DEFDISP
{
	GETDATA;

	if (data->tbuf)
	{
		free(data->tbuf);
	}
	return (DOSUPER);
}


DEFSET
{
	GETDATA;
	BOOL refresh = FALSE;

	FORTAG(INITTAGS)
	{
		case MA_Paneldrag_LockMode:
		{
			refresh = TRUE;

			if (tag->ti_Data && !data->locked)
			{
				data->locked = TRUE;
				DoMethod(obj, MM_Paneldrag_RemoveDragGad);
			}
			else if (!tag->ti_Data && data->locked)
			{
				data->locked = FALSE;
				DoMethod(obj, MM_Paneldrag_AddDragGad);
			}
		}
		break;

		case MA_Paneldrag_Zipping:
		{
			refresh = TRUE;

			if (tag->ti_Data)
			{
				if (!data->locked)
				{
					DoMethod(obj, MM_Paneldrag_RemoveDragGad);
				}
				data->zipping = TRUE;
			}
			else
			{
				if (!data->locked)
				{
					DoMethod(obj, MM_Paneldrag_AddDragGad);
				}
				data->zipping = FALSE;
			}
		}
		break;
	}
	NEXTTAG

	/*  if we don't do this we'll get gfx mess when the user uses a horizontal gradient
	 *  in a horizontal panel or vertical gradient in a vertical panel.
	 */
	if (refresh)
	{
		MUI_Redraw(obj, MADF_DRAWOBJECT);
	}

	return (DOSUPER);
}


DEFMMETHOD(AskMinMax)
{
	ULONG size = getv(_parent(obj), MA_Panelgroup_Size);
	
	DOSUPER;

	ASSERT(size);

	if (getv(_parent(obj), MA_Panelgroup_Horiz))
	{
		msg->MinMaxInfo->MinWidth += MINEMPTY;
		msg->MinMaxInfo->MinHeight += size;
		msg->MinMaxInfo->MaxWidth += MINEMPTY;
		msg->MinMaxInfo->MaxHeight += size;
	}
	else
	{
		msg->MinMaxInfo->MinWidth += size;
		msg->MinMaxInfo->MinHeight += MINEMPTY;
		msg->MinMaxInfo->MaxWidth += size;
		msg->MinMaxInfo->MaxHeight += MINEMPTY;
	}
	return (0);
}


DEFMMETHOD(Setup)
{
	ULONG rc;
	GETDATA;

	if ( (rc = DOSUPER && _win(obj)) )
	{
		data->ehnode.ehn_Object = obj;
		data->ehnode.ehn_Class = cl;
		data->ehnode.ehn_Events = IDCMP_GADGETDOWN | IDCMP_GADGETUP | IDCMP_MOUSEBUTTONS;
		data->ehnode.ehn_Priority = 1; /* XXX: ok ? */
		data->ehnode.ehn_Flags = MUI_EHF_GUIMODE;
		DoMethod(_win(obj), MUIM_Window_AddEventHandler, (ULONG)&data->ehnode);
	}
	return (rc);
}


DEFMMETHOD(HandleEvent)
{
	if (msg->imsg)
	{
		GETDATA;

		switch (msg->imsg->Class)
		{
			case IDCMP_GADGETDOWN:
				{
					if (_isinobject(msg->imsg->MouseX, msg->imsg->MouseY))
					{
						data->oldx = _window(obj)->LeftEdge;
						data->oldy = _window(obj)->TopEdge;
						
						data->objdown = obj;
						
						data->selected = TRUE;
						MUI_Redraw(obj, MADF_DRAWOBJECT);
					}
				}
				break;

			case IDCMP_GADGETUP:
				{				
					if (data->objdown == obj)
					{
						data->selected = FALSE;
						MUI_Redraw(obj, MADF_DRAWOBJECT);
					
						if (data->oldx != _window(obj)->LeftEdge || data->oldy != _window(obj)->TopEdge)
						{
							DoMethod(_win(obj),	MM_Panelwin_AttachToBorder, TRUE);
						}
						else
						{
							DoMethod(_parent(obj), MM_Panelgroup_ToggleZip, data->type);
						}
						data->objdown = NULL;
					}
				}
				break;

			case IDCMP_MOUSEBUTTONS:
				switch (msg->imsg->Code)
				{
					case SELECTDOWN:
						{
							if (_isinobject(msg->imsg->MouseX, msg->imsg->MouseY) && !data->zipping)
							{
								data->objdown = obj;

								data->selected = TRUE;
								MUI_Redraw(obj, MADF_DRAWOBJECT);
							}
						}
						break;
					
					case SELECTUP:
						{
							if (!data->zipping)
							{
								if (data->objdown == obj)
								{
									data->selected = FALSE;
									MUI_Redraw(obj, MADF_DRAWOBJECT);

									if (_isinobject(msg->imsg->MouseX, msg->imsg->MouseY))
									{
										DoMethod(_parent(obj), MM_Panelgroup_ToggleZip, data->type);
									}
								}
								data->objdown = NULL;
							}
						}
						break;
				}
				break;
		}
	}
	return (0);
}


DEFMMETHOD(Cleanup)
{
	GETDATA;

	if (muiRenderInfo(obj) && _win(obj))
	{
		DoMethod(_win(obj), MUIM_Window_RemEventHandler, (ULONG)&data->ehnode);
	}
	return (DOSUPER);
}


DEFTMETHOD(Paneldrag_AddDragGad)
{
	GETDATA;

	if (!data->draggad)
	{
		if ( (data->draggad = NewObject(0, BUTTONGCLASS,
			GA_Left,      _left(obj),
			GA_Top,       _top(obj),
			GA_Width,     _width(obj),
			GA_Height,    _height(obj),
			GA_SysGType,  GTYP_WDRAGGING2,
			GA_Immediate, TRUE,
			TAG_DONE)) )
		{
			AddGList(_window(obj), data->draggad, -1, -1, NULL);
		}
	}
	return (0);
}


DEFTMETHOD(Paneldrag_RemoveDragGad)
{
	GETDATA;

	if (data->draggad)
	{
		RemoveGadget(_window(obj), data->draggad);
		DisposeObject(data->draggad);
		data->draggad = NULL;
	}
	return (0);
}


DEFMMETHOD(Show)
{
	ULONG rc;

	if ( (rc = DOSUPER) )
	{
		GETDATA;

		/*
		 * This is the cleanest way. Other options are:
		 * - EventHandler + MoveWindow(): doesn't work
		 *   because intuition delays events for the next
		 *   intuitick. It becomes jerky then.
		 *
		 * - PeekQualifier() and check if LMB is pressed
		 *   50 times per seconds. This works OK but is not
		 *   very friendly.
		 *
		 * - Current way: system friendly but intuition had
		 *   to get some support for it :)
		 */
		if (!data->locked && !data->zipping)
		{
			DoMethod(obj, MM_Paneldrag_AddDragGad);
		}
	}
	return (rc);
}


DEFMMETHOD(Hide)
{
	DoMethod(obj, MM_Paneldrag_RemoveDragGad);

	return (DOSUPER);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Panel_Type:
			*msg->opg_Storage = MV_Panel_Type_Drag;
			return (TRUE);
	}
	return (DOSUPER);
}


DEFMMETHOD(Draw)
{
	#define MAXVECTORS 5
	GETDATA;
	struct AreaInfo ai;
	struct TmpRas tr;
	ULONG x, y;
	ULONG size;
	UBYTE buf[MAXVECTORS * 5];
	ULONG horiz;
	ULONG rc;

	/*
	 * The following should probably go to
	 * a 'vector' class and be double
	 * buffered also.
	 */
	
	rc = DOSUPER;

	if (msg->flags & (MADF_DRAWOBJECT|MADF_DRAWUPDATE))
	{
		ULONG bgbrightness = gfx_analyze_average_brightness( _rp(obj), _left(obj), _top(obj), _width(obj) , _height(obj) );

		/*
		 * Adjust background width brightening/darkening and draw simple frame
		 */

		ProcessPixelArray(_rp(obj), _left(obj) + 1, _top(obj) + 1, _width(obj) - 1, _height(obj) - 1,  bgbrightness > 128 ? POP_DARKEN : POP_BRIGHTEN, 45, NULL);
		ProcessPixelArray(_rp(obj), _left(obj), _top(obj), _width(obj), 1, data->selected ? POP_DARKEN : POP_BRIGHTEN, 70, NULL);
		ProcessPixelArray(_rp(obj), _left(obj) + _width(obj) - 1, _top(obj), 1, _height(obj), !data->selected ? POP_DARKEN : POP_BRIGHTEN, 70, NULL);
		ProcessPixelArray(_rp(obj), _left(obj), _top(obj) + 1, 1, _height(obj) , data->selected ? POP_DARKEN : POP_BRIGHTEN, 70, NULL);
		ProcessPixelArray(_rp(obj), _left(obj), _top(obj) + _height(obj) - 1, _width(obj), 1 , !data->selected ? POP_DARKEN : POP_BRIGHTEN, 70, NULL);

		horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

		if (horiz)
		{
			if (data->type == MV_Paneldrag_Type_LeftUp)
			{
				x = _left(obj) + (_right(obj) - _left(obj)) / 2 - 2;
			}
			else
			{
				x = _left(obj) + (_right(obj) - _left(obj)) / 2 + 2;
			}
			y = _top(obj) + (_bottom(obj) - _top(obj)) / 2;
		}
		else
		{
			x = _left(obj) + (_right(obj) - _left(obj)) / 2;
			
			if (data->type == MV_Paneldrag_Type_LeftUp)
			{
				y = _top(obj) + (_bottom(obj) - _top(obj)) / 2 - 2;
			}
			else
			{
				y = _top(obj) + (_bottom(obj) - _top(obj)) / 2 + 2;
			}
		}

		if (data->selected) /* XEN selected effect :) */
		{
			x++;
			y++;
		}

		size = RASSIZE(_width(obj), _height(obj));

		if (!data->tbuf || data->oldbufsize != size)
		{
			if (data->tbuf)
			{
				free(data->tbuf);
			}
			data->tbuf = malloc(size);
			data->oldbufsize = size;
		}

		if (data->tbuf)
		{
			InitArea(&ai, buf, MAXVECTORS);
			_rp(obj)->AreaInfo = &ai;
			InitTmpRas(&tr, data->tbuf, size);
			_rp(obj)->TmpRas = &tr;
		
			if ( bgbrightness > 128 )
				SetAPen(_rp(obj), _pens(obj)[MPEN_SHADOW]);
			else
				SetAPen(_rp(obj), _pens(obj)[MPEN_SHINE]);

			
			AreaMove(_rp(obj), x, y);
			
			if (horiz)
			{
				if (data->type == MV_Paneldrag_Type_LeftUp)
				{
					AreaDraw(_rp(obj), x + 4, y - 4);
					AreaDraw(_rp(obj), x + 4, y + 4);
				}
				else
				{
					AreaDraw(_rp(obj), x - 4, y - 4);
					AreaDraw(_rp(obj), x - 4, y + 4);
				}
			}
			else
			{
				if (data->type == MV_Paneldrag_Type_LeftUp)
				{
					AreaDraw(_rp(obj), x + 4, y + 4);
					AreaDraw(_rp(obj), x - 4, y + 4);
				}
				else
				{
					AreaDraw(_rp(obj), x + 4, y - 4);
					AreaDraw(_rp(obj), x - 4, y - 4);
				}
			}
			AreaDraw(_rp(obj), x, y);
			AreaEnd(_rp(obj));
		}
	}

	return rc;
}


BEGINMTABLE
DECNEW
DECDISP
DECSET
DECMMETHOD(AskMinMax)
DECMMETHOD(Setup)
DECMMETHOD(Cleanup)
DECMMETHOD(HandleEvent)
DECMMETHOD(Show)
DECMMETHOD(Hide)
DECTMETHOD(Paneldrag_AddDragGad)
DECTMETHOD(Paneldrag_RemoveDragGad)
DECGET
DECMMETHOD(Draw)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Area, paneldragclass)
