/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: paneltraypanelclass.c,v 1.3 2006/12/20 13:34:15 fab Exp $
 */

#include "ambient.h"

#if USE_TRAY_LIB

/* public */
#include <cybergraphx/cybergraphics.h>
#include <proto/cybergraphics.h>
#include <proto/graphics.h>
#include <proto/gadtools.h>

/* private */
#include "mui_func.h"
#include "rexx.h"
#include "command.h"
#include "contextmenu.h"
#include "iconio.h"
#include "name.h"
#include "gfx_scale.h"
#include "gfx_bitmap.h"
#include "gfx_blit.h"
#include "gfx_pen.h"
#include "screen.h"
#include "wbstart.h"
#include "prefs.h"
#include "datatypes_picture.h"
#include "wbstart.h"
#include "threads.h"
#include "file_func.h"
#include "dragdrop.h"
#include "legacy.h"

#define USE_INLINE_STDARG 1

#include <proto/tray.h>

enum
{
	BMEFFECT_NONE = 0,      /* XXX: from icondata.h, maybe something more global could be invented... */
	BMEFFECT_SELECTED,
	BMEFFECT_MOUSEOVER,
	BMEFFECT_DRAGDROP
};

struct applet_node
{
	struct MinNode node;
	APTR object;
};

struct list_node
{
	struct MinNode node;
	STRPTR applet_name;
};

struct Data {
	ULONG distance;
	APTR cmenu;
	STRPTR help;
	ULONG directory;
	struct MUI_EventHandlerNode ehnode;
	APTR bm;
	APTR sclbm;
	ULONG oldwidth;
	ULONG oldheight;
	#if USE_DTICONS
	APTR dtdtp;
	#endif


	APTR b_size;
	APTR g_size;

	ULONG frame_w;
	ULONG frame_h;

	ULONG width;
	ULONG height;

	BOOL sizing;
	WORD sizing_spx;
	WORD sizing_spy;

	ULONG resize_id;

	struct MinList applets;

	APTR appletswin;
	/* image highlighting */

	ULONG highlighted, pressed;
	ULONG droptarget;
	ULONG current_effect, desired_effect;
	APTR tempbm;
};

static void prefs2list(APTR appletswin, ULONG list, STRPTR data);

static void prefs2list(APTR appletswin, ULONG list, STRPTR data)
{
	int i = 0;

	while (data[i])
	{
		DoMethod(appletswin, MM_Appletswin_AddApplet, list, &data[i]);

		i += strlen(&data[i]) + 1;
	}       
}

static void doset(APTR obj, struct Data *data, struct TagItem *tags)
{
	ULONG update = 0;

	FORTAG(tags)
	{
		case MA_Panel_Distance:
			data->distance = tag->ti_Data;
			break;

		case MA_TrayPanel_WhiteList:
			prefs2list(data->appletswin, MV_Appletswin_WhiteList, (STRPTR)tag->ti_Data);
			break;

		case MA_TrayPanel_BlackList:
			prefs2list(data->appletswin, MV_Appletswin_BlackList, (STRPTR)tag->ti_Data);
			break;

	}
	NEXTTAG 
}


DEFNEW
{
	struct Data *data;
	APTR b_size;
	APTR g_size;
	APTR strip;

	obj = DoSuperNew(cl, obj,
		GroupFrame,
		MUIA_Group_Horiz, TRUE,
		MUIA_InnerLeft, 0,
		MUIA_InnerRight, 0,
		MUIA_InnerTop, 0,
		MUIA_InnerBottom, 0,
		MUIA_ContextMenu, 1,
		MUIA_Group_HorizCenter, 2,
        Child, RectangleObject, MUIA_Weight, 1, End,
		Child, g_size = VGroup, MUIA_FixWidth, 8, MUIA_Weight, 100,
			Child, b_size = GroupObject, ButtonFrame, MUIA_Background, MUII_ButtonBack, MUIA_ShowMe, FALSE, End,
			Child, RectangleObject, MUIA_Weight, 1, End,
		End
	);

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);

	data->help = NULL;
	data->droptarget = FALSE;
	data->b_size = b_size;
	data->g_size = g_size;
	data->sizing = FALSE;
	data->width = 0;
	data->height = 0;
    data->resize_id = 0;

	if ((data->appletswin = NewObject(getappletswinclass(), NULL, TAG_DONE)))
		DoMethod(app, OM_ADDMEMBER, data->appletswin);

	NEWLIST(&data->applets);

	doset(obj, data, INITTAGS);

	return ((ULONG)obj);
}


DEFDISP
{
	struct applet_node *n, *nn;
	GETDATA;

	gfx_bitmap_delete(data->sclbm);
	gfx_bitmap_delete(data->bm);

	if (data->help)
	{
		name_delete(data->help);
	}

	gfx_bitmap_delete(data->tempbm);

	ITERATELISTSAFE(n, nn, &data->applets)
	{
		APTR parent = (APTR)getv(n->object, MUIA_Parent);
		if (parent && DoMethod(parent, MUIM_Group_InitChange))
		{
			DoMethod(parent, OM_REMMEMBER, n->object);

			DoMethod(parent, MUIM_Group_ExitChange);
		}

		free(n);
	}

	return (DOSUPER);
}


DEFMMETHOD(AskMinMax)
{
	ULONG horiz;
	ULONG w = 10;
	ULONG wm = 0;
	ULONG size;
	GETDATA;

	DOSUPER;

	size = getv(_parent(obj), MA_Panelgroup_Size);
	ASSERT(size);

	horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

	FORCHILD(obj, MUIA_Group_ChildList)
	{
		if (horiz)
		{
			if (_minwidth(child) > 0)
			{
				w += max((_maxwidth(child) > size)?size:_maxwidth(child), _minwidth(child));
				wm += _maxwidth(child);
			}
		}
		else
		{
			if (_minheight(child) > 0)
			{
				w += _minheight(child);
				wm += _maxheight(child);
			}
		}
	}
	NEXTCHILD

	if (w == 0)
		w = size;

	if (horiz)
	{
		if (data->width > w)
			w = data->width;
		else
			data->width = w;
	}
	else
	{
		if (data->height > w)
			w = data->height;
		else
			data->height = w;
	}


	if (wm < w)
		wm = w;

	if (horiz)
	{
		msg->MinMaxInfo->MinWidth = w;
		msg->MinMaxInfo->MinHeight = size;
		msg->MinMaxInfo->MaxWidth = wm;
		msg->MinMaxInfo->MaxHeight = size;
	}
	else
	{
		msg->MinMaxInfo->MinWidth = size;
		msg->MinMaxInfo->MinHeight = w;
		msg->MinMaxInfo->MaxWidth = size;
		msg->MinMaxInfo->MaxHeight = wm;
	}

	return (0);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Panel_Type:
			*msg->opg_Storage = MV_Panel_Type_TrayPanel;
			return (TRUE);
			break;

		case MA_Panel_Distance:
			{
				GETDATA;
				*msg->opg_Storage = data->distance;
			}
			return (TRUE);
			break;

		case MA_Panel_Properties:
            *msg->opg_Storage = TRUE;
			return (TRUE);
			break;
	}
	return (DOSUPER);
}


DEFSET
{
	GETDATA;

	doset(obj, data, INITTAGS);

	return (DOSUPER);
}


DEFMMETHOD(Setup)
{
	GETDATA;
	ULONG rc;

	rc = DOSUPER;

	set(obj, MUIA_Group_Horiz, getv(_parent(obj), MA_Panelgroup_Horiz));
	if (rc && _win(obj))
	{
		data->ehnode.ehn_Object = obj;
		data->ehnode.ehn_Class = cl;
		data->ehnode.ehn_Events = IDCMP_MOUSEMOVE | IDCMP_MOUSEBUTTONS;
		data->ehnode.ehn_Priority = 1; /* XXX: ok ? */
		data->ehnode.ehn_Flags = MUI_EHF_GUIMODE;
		DoMethod(_win(obj), MUIM_Window_AddEventHandler, (ULONG)&data->ehnode);
	}

	return (rc);
}

DEFMMETHOD(Cleanup)
{
	GETDATA;

	if (muiRenderInfo(obj) && _win(obj))
	{
		DoMethod(_win(obj), MUIM_Window_RemEventHandler, (ULONG)&data->ehnode);
	}

	return (DOSUPER);
}

DEFMMETHOD(Draw)
{
	GETDATA;

	data->frame_w = _addleft(obj);
	data->frame_h = _addtop(obj);
	data->width = _width(obj);
	data->height = _height(obj);

	return (DOSUPER);
}

DEFMMETHOD(ContextMenuAdd)
{
	ULONG flags = 0;
	GETDATA;

	if (!getv(_parent(obj), MA_Panelgroup_Locked))
		flags |= AS_PANEL_UNLOCKED;

	data->cmenu	= contextmenu_build_simple(CM_PANELTRAYPANEL, flags);

	if (data->cmenu)
		DoMethod(msg->menustrip, OM_ADDMEMBER, data->cmenu);

	return (DOSUPER);
}


DEFMMETHOD(ContextMenuChoice)
{
	GETDATA;
	struct command_menu *cm;
	ULONG rc = 0;

	if (data->cmenu && DoMethod(data->cmenu, MUIM_FindObject, msg->item))
	{
		if ( ((cm = (struct command_menu *)getv(msg->item, MA_Menuitem_Command))) )
		{
			if (cm->name && *cm->name)
			{
				execute_command(_parent(obj), cm->type, cm->args, NULL);
			}
		}
	}
	else
		rc = DOSUPER;

	data->cmenu = NULL;

	return (rc);
}


DEFSMETHOD(Thread_Finished)
{
	return (0);
}

DEFMMETHOD(DragDrop)
{
	if (DoMethod(app, MM_Application_DetachTrayObject, msg->obj))
		DoMethod(obj, MM_TrayPanel_AttachObject, msg->obj, NULL);

	return (0);
}

/*
 * Prevents areaclass from drawing a stupid frame around.
 */
#if 1
DEFMMETHOD(DragBegin)
{

	GETDATA;

	if ( msg->obj != obj )
	{
		if ( getv(_parent(obj), MA_Panelgroup_Locked) )
		{
			data->droptarget = TRUE;
			data->desired_effect = BMEFFECT_DRAGDROP;
			data->current_effect = -1;
			MUI_Redraw(obj, MADF_DRAWOBJECT);
			return (0);
		}
	}

	return(DOSUPER);
}

DEFMMETHOD(DragFinish)
{
	GETDATA;

	if ( msg->obj != obj )
	{
		if ( getv(_parent(obj), MA_Panelgroup_Locked) )
		{
			data->droptarget = FALSE;
			data->desired_effect = 0;
			data->current_effect = -1;
			MUI_Redraw(obj, MADF_DRAWOBJECT);
			return (0);
		}
	}

	return (DOSUPER);
}

#endif

DEFMMETHOD(HandleEvent)
{
	GETDATA;

	if (msg->imsg)
	{
		switch (msg->imsg->Class)
		{
			case IDCMP_MOUSEMOVE:
			{
				GETDATA;

				WORD x, y;
				ULONG w, h;
				ULONG horiz;

				horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

				x = msg->imsg->MouseX - getv(obj, MUIA_LeftEdge);
				y = msg->imsg->MouseY - getv(obj, MUIA_TopEdge);
				w = getv(obj, MUIA_Width);
				h = getv(obj, MUIA_Height);

				if (!data->sizing)
				{
					if (horiz)
					{
						if ((x < w - data->frame_w) && (x > w - 10 - data->frame_w) && (y > 0) && (y < h))
						{
							set(data->b_size, MUIA_ShowMe, TRUE);
						}
						else
							set(data->b_size, MUIA_ShowMe, FALSE);
					}
					else
					{
						if ((y < h - data->frame_h) && (y > h - 10 - data->frame_h) && (x > 0) && (x < w))
						{
							set(data->b_size, MUIA_ShowMe, TRUE);
						}
						else
							set(data->b_size, MUIA_ShowMe, FALSE);
					}
				}
				else
				{
					if (horiz)
						data->width += x - data->sizing_spx;
					else
						data->height += y - data->sizing_spy;

                    data->sizing_spx = x;
					data->sizing_spy = y;


					if (data->resize_id)
						DoMethod(app, MUIM_Application_KillPushMethod, obj, data->resize_id);
					
					data->resize_id = DoMethod(app, MUIM_Application_PushMethod, _parent(obj), (2 | MUIV_PushMethod_Delay(50)) , MM_Panelgroup_Refresh, FALSE);
				}

				return (0);//MUI_EventHandlerRC_Eat); /* do not confuse plugins */

			}
			break;

			case IDCMP_MOUSEBUTTONS:
			{
				switch (msg->imsg->Code)
				{
					case SELECTDOWN:
					{
						GETDATA;
						WORD x,y;
						LONG w,h;
						ULONG horiz;

						horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

						x = msg->imsg->MouseX - getv(obj, MUIA_LeftEdge);
						y = msg->imsg->MouseY - getv(obj, MUIA_TopEdge);
						w = getv(obj, MUIA_Width);
						h = getv(obj, MUIA_Height);

						if (horiz)
						{
							if ((x < w - data->frame_w) && (x > w - 10 - data->frame_w) && (y > 0) && (y < h))
							{
								data->sizing_spx = x;
								data->sizing = TRUE;

								return (MUI_EventHandlerRC_Eat);

							}
						}
						else
						{
							if ((y < h - data->frame_h) && (y > h - 10 - data->frame_h) && (x > 0) && (x < w))
							{
								data->sizing_spy = y;
								data->sizing = TRUE;

								return (MUI_EventHandlerRC_Eat);
							}
						}
                        return (0);
					}
					break;

					case SELECTUP:
					{
						data->sizing = FALSE;
					}
					break;
				}
			}
			break;

			case IDCMP_INACTIVEWINDOW:
			{
				GETDATA;

				DB(("INACTIVE WINDOW\n"));
			}
			break;
		}
	}
	return (0);
}


DEFSMETHOD(Panel_SaveConfig)
{
	APTR pl, pi; /* list, item */
	GETDATA;
	ASSERT(msg->pctx);

	if (!(pl = prefspool_item_get(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL)))
	{
		pl = prefspool_item_add(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL);
	}

	if (pl)
	{
		if (!(pi = prefspool_item_get(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL)))
		{
			pi = prefspool_item_add(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL);
		}
		if (pi)
		{
			setprefslong_lp(msg->pctx, pi, DSI_LISTPOOL_PANEL_DISTANCE, data->distance);
			{
				STRPTR list;
				ULONG len;

				/* Save white list */
				DoMethod(data->appletswin, MM_Appletswin_GetList, MV_Appletswin_WhiteList, (ULONG)&list, (ULONG)&len);
				prefspool_item_add(msg->pctx, pi, DSI_LISTPOOL_TRAYPANEL_WHITELIST, list, len);

				/* Save black list */
				DoMethod(data->appletswin, MM_Appletswin_GetList, MV_Appletswin_BlackList, (ULONG)&list, (ULONG)&len);
				prefspool_item_add(msg->pctx, pi, DSI_LISTPOOL_TRAYPANEL_BLACKLIST, list, len);
			}
		}
	}
	
	return (0);
}


DEFSMETHOD(TrayPanel_AttachObject)
{
	GETDATA;
	ULONG res = 0;

	if (msg->object)
	{
		APTR group;

		if (msg->name)
			res = DoMethod(data->appletswin, MM_Appletswin_AddApplet, MV_Appletswin_WhiteList, msg->name);
		else
			res = TRUE;

		if (res)
		{
			group = VGroup, MUIA_Weight, 1000, Child, msg->object, Child, RectangleObject, MUIA_Weight, 1, End, End;

			if (group && DoMethod(obj, MUIM_Group_InitChange))
			{
				struct applet_node *node;
				struct list_node *node2;

				DoMethod(obj, OM_ADDMEMBER, group);
	
				DoMethod(obj, MUIM_Group_MoveMember, data->g_size, -1);

				DoMethod(obj, MUIM_Group_ExitChange);

				node = malloc(sizeof(struct applet_node));
				node->object = (APTR)msg->object;

				ADDTAIL(&data->applets, node);
			}
		}
	}

	return (res);
}

DEFSMETHOD(TrayPanel_DetachObject)
{
	GETDATA;
	struct applet_node *n, *nn;

	ITERATELISTSAFE(n, nn, &data->applets)
	{
		if (n->object == (APTR)msg->object)
		{
			APTR parent = _parent((APTR)msg->object);

			if (parent && DoMethod(parent, MUIM_Group_InitChange))
			{
				APTR parentparent;

                DoMethod(parent, OM_REMMEMBER, msg->object);
				DoMethod(parent, MUIM_Group_ExitChange);

				parentparent = _parent(parent);

				if (parentparent && DoMethod(parentparent, MUIM_Group_InitChange))
				{
					DoMethod(parentparent, OM_REMMEMBER, parent);
					DoMethod(parentparent, MUIM_Group_ExitChange);
					MUI_DisposeObject(parent);
				}
			}

			REMOVE(n);
			free(n);

			return (TRUE);
		}
	}

	return (FALSE);
}

DEFSMETHOD(TrayPanel_ManageApplets)
{
	GETDATA;

	if (data->appletswin)
	{
		set(data->appletswin, MUIA_Window_Open, TRUE);
	}


	return (0);
}

DEFMMETHOD(DragQuery)
{
	if (_win(msg->obj) != _win(obj))
	{
		if (getv(_win(msg->obj), MA_Window_Type) == MV_Window_Type_Panel)
			return (MUIV_DragQuery_Accept);
	}

	return (0);
}

DEFMMETHOD(Group_ExitChange)
{
	ULONG horiz;
	GETDATA;

	horiz = getv(_parent(obj), MA_Panelgroup_Horiz);

	set(obj, MUIA_Group_Horiz, horiz);

	DoMethod(data->g_size, MUIM_Group_InitChange);
	SetAttrs(data->g_size, MUIA_FixHeight, horiz ? 0 : 8, MUIA_FixWidth, horiz ? 8 : 0, TAG_DONE);
	DoMethod(data->g_size, MUIM_Group_ExitChange);

	return (DOSUPER);
}


BEGINMTABLE
DECNEW
DECDISP
DECGET
DECSET
DECMMETHOD(AskMinMax)
DECMMETHOD(Setup)
DECMMETHOD(Cleanup)
DECMMETHOD(Draw)
DECMMETHOD(HandleEvent)
DECMMETHOD(ContextMenuChoice)
DECMMETHOD(ContextMenuAdd)
DECMMETHOD(DragQuery)
DECMMETHOD(DragBegin)
DECMMETHOD(DragFinish)
DECMMETHOD(DragDrop)
DECSMETHOD(Panel_SaveConfig)
DECSMETHOD(TrayPanel_AttachObject)
DECSMETHOD(TrayPanel_DetachObject)
DECSMETHOD(TrayPanel_ManageApplets)
DECSMETHOD(Thread_Finished)
DECMMETHOD(Group_ExitChange)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Virtgroup, traypanelclass)

#endif
