/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: prefswin_keyboardclass.c,v 1.4 2007/07/08 20:56:42 fab Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "ambient_cat.h"
#include "mui_func.h"
#include "prefswin.h"
#include "prefsclone.h"
#include "time_func.h"
#include "keyshortcuts.h"
#include "command.h"

struct Data {
	APTR group_builtin_shortcuts;
	APTR group_custom_shortcuts;
};

DEFNEW
{
	struct Data *data;
	APTR group_builtin_shortcuts;
	APTR group_custom_shortcuts;
	APTR o;
	APTR bt_addshortcut;
	struct key_shortcut_node * n;
	
    obj = DoSuperNew(cl, obj,
		Child,
		ScrollgroupObject,
			//MUIA_Weight, 60,
			MUIA_Scrollgroup_FreeVert, TRUE,
			MUIA_Scrollgroup_AutoBars, TRUE,
			GroupFrameT(GSI(MSG_PREFSWIN_KEYBOARDCLASS_BUILTINGROUP)),
			MUIA_Scrollgroup_Contents,
			group_builtin_shortcuts	=  VirtgroupObject, End,
		End,

		Child,
		ScrollgroupObject,
			//MUIA_Weight, 40,
			MUIA_Scrollgroup_FreeVert, TRUE,
			MUIA_Scrollgroup_AutoBars, TRUE,
			GroupFrameT( GSI(MSG_PREFSWIN_KEYBOARDCLASS_CUSTOMGROUP)),
			MUIA_Scrollgroup_Contents,
			group_custom_shortcuts =  VirtgroupObject, End,
		End,

		Child, HGroup,
			Child, HSpace(0),
			Child, bt_addshortcut = MUICreateButton(MSG_PREFSWIN_KEYBOARDCLASS_ADDAHOTKEY, NULL),
			Child, HSpace(0),
			End,
	End;

	if (!obj)
	{
		return (0);
	}

	data = INST_DATA(cl, obj);

	data->group_builtin_shortcuts = group_builtin_shortcuts;
	data->group_custom_shortcuts = group_custom_shortcuts;

	/* Add builtin shortcuts */
	n = keyshortcuts_next(NULL);

	do
	{
		if(n && (n->shortcut.flags & SHORTCUT_FLAG_BUILTIN) && (n->shortcut.flags & SHORTCUT_FLAG_ENABLED) )
		{
			o = NewObject(getkeyshortcutclass(), NULL,
							MA_KeyShortcut_Shortcut, &n->shortcut,
							MA_KeyShortcut_EditAction, FALSE,
                            MA_Prefswin_Object, obj,
							End;

			DoMethod(data->group_builtin_shortcuts, MUIM_Group_InitChange);
			DoMethod(data->group_builtin_shortcuts, OM_ADDMEMBER, o);
			DoMethod(data->group_builtin_shortcuts, MUIM_Group_ExitChange);
		}

		n = keyshortcuts_next(n);

	} while(n);

	/* Add custom shortcuts */
	n = keyshortcuts_next(NULL);

	do
	{
		if(n && !(n->shortcut.flags & SHORTCUT_FLAG_BUILTIN) && (n->shortcut.flags & SHORTCUT_FLAG_ENABLED) )
		{
			o = NewObject(getkeyshortcutclass(), NULL,
							MA_KeyShortcut_Shortcut, &n->shortcut,
							MA_KeyShortcut_EditAction, TRUE,
                            MA_Prefswin_Object, obj,
							End;

			DoMethod(data->group_custom_shortcuts, MUIM_Group_InitChange);
			DoMethod(data->group_custom_shortcuts, OM_ADDMEMBER, o);
			DoMethod(data->group_custom_shortcuts, MUIM_Group_ExitChange);
		}

		n = keyshortcuts_next(n);

	} while(n);

	DoMethod(data->group_custom_shortcuts, MUIM_Group_InitChange);
	o = RectangleObject, End;

	if (o)
	{
		DoMethod(data->group_custom_shortcuts, OM_ADDMEMBER, o);
	}

    DoMethod(data->group_custom_shortcuts, MUIM_Group_ExitChange);


	set(bt_addshortcut, MUIA_Weight, 10);

	DoMethod(bt_addshortcut, MUIM_Notify, MUIA_Pressed, FALSE,
		obj, 1, MM_Prefswin_Keyboard_AddShortcut
	);


	return ((ULONG)obj);
}

DEFTMETHOD(Prefswin_Store)
{
	GETDATA;
	struct key_shortcut_t * shortcut;
	TEXT buffer[128];
	APTR pl, pi;
	ULONG i;

	prefspool_item_remove(cloneprefspool, NULL, DSI_LISTPOOL_KEYSHORTCUT);

	pl = prefspool_item_get(cloneprefspool, NULL, DSI_LISTPOOL_KEYSHORTCUT, NULL, NULL);

	if (!pl)
	{
		pl = prefspool_item_add(cloneprefspool, NULL, DSI_LISTPOOL_KEYSHORTCUT, NULL, NULL);
	}

	if(pl)
	{
		i = 0;
		/*  builtin commands */
		FORCHILD(data->group_builtin_shortcuts, MUIA_Group_ChildList)
		{
			APTR ptr = (APTR) getv(child, MA_KeyShortcut_Shortcut);

			shortcut = (struct key_shortcut_t *) ptr;

			if (shortcut)
			{
				struct key_shortcut_node * n;

				if(shortcut->name && shortcut->action.command)
				{
					keyshortcut_sequence_to_string(&shortcut->sequence, buffer, sizeof(buffer));

					if (!(pi = prefspool_item_get(cloneprefspool, pl, i | DSF_LISTPOOL, NULL, NULL)))
					{
						pi = prefspool_item_add(cloneprefspool, pl, i | DSF_LISTPOOL, NULL, NULL);
					}

					if (pi)
					{
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_NAME, shortcut->name);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_MSGID, shortcut->msgid);
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_DEFINITION, buffer);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_ID, shortcut->id);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_FLAGS, shortcut->flags);
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_CMDSTRING, shortcut->action.command);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_CMDTYPE, shortcut->action.type);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_CMDFLAGS, shortcut->action.flags);
					}

					i++;
				}
			}
		}
		NEXTCHILD

		/* custom commands */

		FORCHILD(data->group_custom_shortcuts, MUIA_Group_ChildList)
		{
			APTR ptr = (APTR) getv(child, MA_KeyShortcut_Shortcut);

			shortcut = (struct key_shortcut_t *) ptr;

			if (shortcut && stricmp(OCLASS(child)->cl_ID, "Rectangle.mui"))  /* why does rectangle also return a pointer ? */
			{
				struct key_shortcut_node * n;

				if(shortcut->name && shortcut->action.command)
				{
					keyshortcut_sequence_to_string(&shortcut->sequence, buffer, sizeof(buffer));

					if (!(pi = prefspool_item_get(cloneprefspool, pl, i | DSF_LISTPOOL, NULL, NULL)))
					{
						pi = prefspool_item_add(cloneprefspool, pl, i | DSF_LISTPOOL, NULL, NULL);
					}

					if (pi)
					{
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_NAME, shortcut->name);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_MSGID, shortcut->msgid);
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_DEFINITION, buffer);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_ID, shortcut->id);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_FLAGS, shortcut->flags);
						setprefsstr_lp(cloneprefspool,  pi, DSI_LISTPOOL_KEYSHORTCUT_CMDSTRING, shortcut->action.command);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_CMDTYPE, shortcut->action.type);
						setprefslong_lp(cloneprefspool, pi, DSI_LISTPOOL_KEYSHORTCUT_CMDFLAGS, shortcut->action.flags);
					}

					i++;
				}
			}
		}
		NEXTCHILD
	}

	setprefslong(DSI_LISTPOOL_KEYSHORTCUT_CHANGED, timed());

	return (0);
}

DEFDISPOSE
{
	DoMethod(obj, MM_Prefswin_Store);

	return (DOSUPER);
}

DEFTMETHOD(Prefswin_Keyboard_AddShortcut)
{
	GETDATA;
	struct key_shortcut_t * shortcut;
	APTR o;

	shortcut = keyshortcut_create( GSI(MSG_PREFSWIN_KEYBOARDCLASS_NEWSHORTCUT) );

	if(shortcut)
	{
		APTR spacer = NULL;

		FORCHILD(data->group_custom_shortcuts, MUIA_Group_ChildList)
		{
			if (!stricmp(OCLASS(child)->cl_ID, "Rectangle.mui")) /* sigh, why is it needed */
			{
				spacer = child;
				break;
			}
		}
		NEXTCHILD

		if(DoMethod(data->group_custom_shortcuts, MUIM_Group_InitChange))
		{
			o = NewObject(getkeyshortcutclass(), NULL,
							MA_KeyShortcut_Shortcut, shortcut,
							MA_KeyShortcut_EditAction, TRUE,
							MA_Prefswin_Object, obj,
							End;


			if(spacer)
				DoMethod(data->group_custom_shortcuts, OM_REMMEMBER, spacer);

			DoMethod(data->group_custom_shortcuts, OM_ADDMEMBER, o);

			if(spacer)
				DoMethod(data->group_custom_shortcuts, OM_ADDMEMBER, spacer);

			DoMethod(data->group_custom_shortcuts, MUIM_Group_ExitChange);
		}

		DoMethod(_parent(data->group_custom_shortcuts), MUIM_Group_InitChange);
		DoMethod(_parent(data->group_custom_shortcuts), MUIM_Group_ExitChange);

		keyshortcut_delete(shortcut);
	}

	return (0);
}

DEFSMETHOD(Prefswin_Keyboard_RemoveShortcut)
{
	GETDATA;

	if(DoMethod(data->group_custom_shortcuts, MUIM_Group_InitChange))
	{
		DoMethod(data->group_custom_shortcuts, OM_REMMEMBER, msg->object);
		DoMethod(_app(obj), MUIM_Application_PushMethod, app, 2, MM_Application_DisposeObject, msg->object);
		DoMethod(data->group_custom_shortcuts, MUIM_Group_ExitChange);
	}

	DoMethod(_parent(data->group_custom_shortcuts), MUIM_Group_InitChange);
	DoMethod(_parent(data->group_custom_shortcuts), MUIM_Group_ExitChange);

	return (0);
}

BEGINMTABLE
DECNEW
DECTMETHOD(Prefswin_Store)
DECTMETHOD(Prefswin_Keyboard_AddShortcut)
DECSMETHOD(Prefswin_Keyboard_RemoveShortcut)
DECDISPOSE
ENDMTABLE

DECSUBCLASS_NC(MUIC_Group, prefswin_keyboardclass)
